// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: Huawei Inc.

// Package cmdrauccustomboothandler implements the rauc-custom-boot-handler program.
package cmdrauccustomboothandler

import (
	"flag"
	"fmt"
	"os"

	"git.ostc-eu.org/OSTC/OHOS/components/sysota/dbusutil"
	"git.ostc-eu.org/OSTC/OHOS/components/sysota/ota/raucinterface"
	"git.ostc-eu.org/OSTC/OHOS/components/sysota/rauc/boothandler"
)

// Name is the name of this command inside the multi-call binary.
const Name = "rauc-custom-boot-handler"

// Run implements the rauc-custom-boot-handler command line tool.
func Run() error {
	fs := flag.NewFlagSet("rauc-custom-boot-handler", flag.ContinueOnError)
	fs.Usage = func() {
		w := fs.Output()
		_, _ = fmt.Fprintf(w, "usage: %s <command> [args]\n", fs.Name())
		_, _ = fmt.Fprintf(w, "\n  command is one of {get,set}-{primary,state}\n")
	}

	if err := fs.Parse(os.Args[1:]); err != nil {
		return err
	}

	if fs.NArg() < 1 {
		fs.Usage()
		return flag.ErrHelp
	}

	cmd := fs.Arg(0)
	args := fs.Args()[1:]

	conn, err := dbusutil.SystemBus()
	if err != nil {
		return err
	}

	defer func() {
		e := conn.Close()
		if err == nil {
			err = e
		}
	}()

	client := raucinterface.NewClient(conn)

	return boothandler.Command(client, cmd, args...)
}
