// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: Huawei Inc.

// Package cmdraucinstallhandler implements the rauc-{pre,post}-install-handler programs.
package cmdraucinstallhandler

import (
	"os"

	"git.ostc-eu.org/OSTC/OHOS/components/sysota/dbusutil"
	"git.ostc-eu.org/OSTC/OHOS/components/sysota/ota/raucinterface"
	"git.ostc-eu.org/OSTC/OHOS/components/sysota/rauc/installhandler"
)

// do calls a function with a D-Bus client and RAUC install handler context.
func do(fn func(client *raucinterface.Client, ctx *installhandler.HandlerContext) error) error {
	ctx, err := installhandler.NewHandlerContext(os.Environ())
	if err != nil {
		return err
	}

	conn, err := dbusutil.SystemBus()
	if err != nil {
		return err
	}

	defer func() {
		e := conn.Close()
		if err == nil {
			err = e
		}
	}()

	client := raucinterface.NewClient(conn)

	return fn(client, ctx)
}

// RunPreInstallHandler implements rauc-pre-install-handler.
//
// The handler captures certain parts of the environment relevant to the handler and
// invokes a D-Bus method on SystemOTA service to complete the request.
func RunPreInstallHandler() error {
	return do(func(client *raucinterface.Client, ctx *installhandler.HandlerContext) error {
		return client.PreInstall(ctx)
	})
}

// RunPostInstallHandler implements rauc-post-install-handler.
//
// The handler captures certain parts of the environment relevant to the handler and
// invokes a D-Bus method on SystemOTA service to complete the request.
func RunPostInstallHandler() error {
	return do(func(client *raucinterface.Client, ctx *installhandler.HandlerContext) error {
		return client.PostInstall(ctx)
	})
}
