// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: Huawei Inc.

// Package main implements the SystemOTA multi-call binary, capable of acting
// like one of several programs.
package main

import (
	"errors"
	"flag"
	"fmt"
	"os"
	"path/filepath"

	"git.ostc-eu.org/OSTC/OHOS/components/sysota/cmd/sysotad/cmdrauccustomboothandler"
	"git.ostc-eu.org/OSTC/OHOS/components/sysota/cmd/sysotad/cmdraucinstallhandler"
	"git.ostc-eu.org/OSTC/OHOS/components/sysota/cmd/sysotad/cmdsysotad"
)

func dispatchCall() error {
	if len(os.Args) == 0 {
		return errors.New("cannot determine program name")
	}

	execFullName := os.Args[0]

	switch execName := filepath.Base(execFullName); execName {
	case "sysotad.test":
		// Name used when testing this package with "go test".
		return nil
	case "sysotad":
		return cmdsysotad.Run(cmdsysotad.DefaultOptions())
	case "rauc-pre-install-handler":
		return cmdraucinstallhandler.RunPreInstallHandler()
	case "rauc-post-install-handler":
		return cmdraucinstallhandler.RunPostInstallHandler()
	case cmdrauccustomboothandler.Name:
		return cmdrauccustomboothandler.Run()
	default:
		return fmt.Errorf("cannot run sysotad with name: %s", execName)
	}
}

func main() {
	if err := dispatchCall(); err != nil {
		if !errors.Is(err, flag.ErrHelp) {
			_, _ = fmt.Fprintf(os.Stderr, "error: %s\n", err)
		}

		os.Exit(1)
	}
}
