// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: Huawei Inc.

// Package dbusutil contains D-Bus utility code not present in godbus.
package dbusutil

import (
	dbus "github.com/godbus/dbus/v5"
)

const (
	// PropertiesInterfaceName is the name of the standard Properties interface.
	PropertiesInterfaceName = "org.freedesktop.DBus.Properties"
	// IntrospectableInterfaceName is the name of the standard Introspectable interface.
	IntrospectableInterfaceName = "org.freedesktop.DBus.Introspectable"
	// PropertiesChangedSignal is the name of the PropertiesChanged signal.
	PropertiesChangedSignal = "org.freedesktop.DBus.Properties.PropertiesChanged"
	// PropertyAnnotationEmitsChangedSignal is a standard annotation for individual properties.
	PropertyAnnotationEmitsChangedSignal = "org.freedesktop.DBus.Property.EmitsChangedSignal"
)

const (
	// In denotes incoming argument in the D-Bus introspection data.
	In = "in"
	// Out denotes outgoing argument in the D-Bus introspection data.
	Out = "out"
)

// SystemBus is like dbus.SystemBus but the returned connection can be closed.
func SystemBus() (conn *dbus.Conn, err error) {
	conn, err = dbus.SystemBusPrivate()
	if err != nil {
		return nil, err
	}

	defer func() {
		if err != nil && conn != nil {
			_ = conn.Close()
		}
	}()

	if err := conn.Auth(nil); err != nil {
		return nil, err
	}

	if err := conn.Hello(); err != nil {
		return nil, err
	}

	return conn, nil
}
