// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: Huawei Inc.

package raucinterface

import (
	dbus "github.com/godbus/dbus/v5"

	"git.ostc-eu.org/OSTC/OHOS/components/sysota/boot"
	"git.ostc-eu.org/OSTC/OHOS/components/sysota/ota"
	"git.ostc-eu.org/OSTC/OHOS/components/sysota/rauc/installhandler"
)

// Client uses the RAUC interface offered by SystemOTA over D-Bus.
type Client struct {
	conn *dbus.Conn
}

// NewClient returns new Client instance using given bus connection.
func NewClient(conn *dbus.Conn) *Client {
	return &Client{conn: conn}
}

// PrimarySlot calls the dev.ostc.sysota1.RAUC.GetPrimary D-Bus method.
func (client *Client) PrimarySlot() (slot boot.Slot, err error) {
	var slotName string

	serviceObj := client.conn.Object(ota.ServiceBusName, ota.ServiceObjectPath)
	if err := serviceObj.Call(InterfaceName+"."+getPrimaryDBusName, 0).Store(&slotName); err != nil {
		return boot.InvalidSlot, err
	}

	err = slot.UnmarshalText([]byte(slotName))

	return slot, err
}

// SetPrimarySlot calls the dev.ostc.sysota1.RAUC.SetPrimary D-Bus method.
func (client *Client) SetPrimarySlot(slot boot.Slot) error {
	serviceObj := client.conn.Object(ota.ServiceBusName, ota.ServiceObjectPath)
	return serviceObj.Call(InterfaceName+"."+setPrimaryDBusName, 0, slot.String()).Store()
}

// SlotState calls the dev.ostc.sysota1.RAUC.GetState D-Bus method.
func (client *Client) SlotState(slot boot.Slot) (state boot.SlotState, err error) {
	var stateName string

	serviceObj := client.conn.Object(ota.ServiceBusName, ota.ServiceObjectPath)
	if err := serviceObj.Call(InterfaceName+"."+getStateDBusName, 0, slot.String()).Store(&stateName); err != nil {
		return boot.InvalidSlotState, err
	}

	err = state.UnmarshalText([]byte(stateName))

	return state, err
}

// SetSlotState calls the dev.ostc.sysota1.RAUC.SetState D-Bus method.
func (client *Client) SetSlotState(slot boot.Slot, state boot.SlotState) error {
	serviceObj := client.conn.Object(ota.ServiceBusName, ota.ServiceObjectPath)
	return serviceObj.Call(InterfaceName+"."+setStateDBusName, 0, slot.String(), state.String()).Store()
}

// PreInstall calls the dev.ostc.sysota1.RAUC.PreInstall D-Bus method.
func (client *Client) PreInstall(ctx *installhandler.HandlerContext) error {
	serviceObj := client.conn.Object(ota.ServiceBusName, ota.ServiceObjectPath)
	return serviceObj.Call(InterfaceName+"."+preInstallDBusName, 0, ctx.Environ()).Store()
}

// PostInstall calls the dev.ostc.sysota1.RAUC.PostInstall D-Bus method.
func (client *Client) PostInstall(ctx *installhandler.HandlerContext) error {
	serviceObj := client.conn.Object(ota.ServiceBusName, ota.ServiceObjectPath)
	return serviceObj.Call(InterfaceName+"."+postInstallDBusName, 0, ctx.Environ()).Store()
}
