// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: Huawei Inc.

package condfilter_test

import (
	"errors"

	. "gopkg.in/check.v1"

	"git.ostc-eu.org/OSTC/OHOS/components/sysota/picfg/condfilter"
)

type allFilterSuite struct {
	filter condfilter.AllFilter
}

var _ = Suite(&allFilterSuite{})

func (s *allFilterSuite) TestMarshalText(c *C) {
	text, err := s.filter.MarshalText()
	c.Assert(err, IsNil)
	c.Check(string(text), Equals, "[all]")
}

func (s *allFilterSuite) TestUnmarshalText(c *C) {
	err := s.filter.UnmarshalText([]byte(`[all]`))
	c.Assert(err, IsNil)

	err = s.filter.UnmarshalText([]byte(`[foo`))
	c.Assert(err, ErrorMatches, `AllFilter "\[foo" error: syntax error`)
	c.Check(errors.Is(err, condfilter.ErrSyntax), Equals, true)

	err = s.filter.UnmarshalText([]byte(`foo]`))
	c.Assert(err, ErrorMatches, `AllFilter "foo\]" error: syntax error`)
	c.Check(errors.Is(err, condfilter.ErrSyntax), Equals, true)

	err = s.filter.UnmarshalText([]byte(`[foo]`))
	c.Assert(err, ErrorMatches, `AllFilter "\[foo\]" error: different filter`)
	c.Check(errors.Is(err, condfilter.ErrDifferent), Equals, true)
}

func (s *allFilterSuite) TestConditionalFilter(c *C) {
	c.Assert(s.filter.ConditionalFilter(), Equals, `[all]`)
}
