// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: Huawei Inc.

package condfilter_test

import (
	"errors"

	. "gopkg.in/check.v1"

	"git.ostc-eu.org/OSTC/OHOS/components/sysota/picfg/condfilter"
)

type edidFilterSuite struct {
	filter condfilter.EDIDFilter
}

var _ = Suite(&edidFilterSuite{})

func (s *edidFilterSuite) TestMarshalText(c *C) {
	edidFilter := condfilter.EDIDFilter("foo")
	text, err := edidFilter.MarshalText()
	c.Assert(err, IsNil)
	c.Check(string(text), Equals, "[EDID=foo]")
}

func (s *edidFilterSuite) TestUnmarshalText(c *C) {
	var paramErr *condfilter.ParameterError

	err := s.filter.UnmarshalText([]byte(`[EDID=value]`))
	c.Assert(err, IsNil)
	c.Check(string(s.filter), Equals, "value")

	err = s.filter.UnmarshalText([]byte(`[EDID=]`))
	c.Assert(err, ErrorMatches, `EDIDFilter "\[EDID=\]" error: invalid monitor EDID: parameter value required`)
	c.Check(errors.Is(err, condfilter.ErrParameterRequired), Equals, true)
	c.Assert(errors.As(err, &paramErr), Equals, true)
	c.Check(paramErr.What, Equals, condfilter.MonitorEDID)

	err = s.filter.UnmarshalText([]byte(`[EDID]`))
	c.Assert(err, ErrorMatches, `EDIDFilter "\[EDID\]" error: different filter`)
	c.Check(errors.Is(err, condfilter.ErrDifferent), Equals, true)

	err = s.filter.UnmarshalText([]byte(`[foo`))
	c.Assert(err, ErrorMatches, `EDIDFilter "\[foo" error: syntax error`)
	c.Check(errors.Is(err, condfilter.ErrSyntax), Equals, true)

	err = s.filter.UnmarshalText([]byte(`foo]`))
	c.Assert(err, ErrorMatches, `EDIDFilter "foo\]" error: syntax error`)
	c.Check(errors.Is(err, condfilter.ErrSyntax), Equals, true)

	err = s.filter.UnmarshalText([]byte(`[foo]`))
	c.Assert(err, ErrorMatches, `EDIDFilter "\[foo\]" error: different filter`)
	c.Check(errors.Is(err, condfilter.ErrDifferent), Equals, true)

	err = s.filter.UnmarshalText([]byte(`[foo=]`))
	c.Assert(err, ErrorMatches, `EDIDFilter "\[foo=\]" error: different filter`)
	c.Check(errors.Is(err, condfilter.ErrDifferent), Equals, true)
}

func (s *edidFilterSuite) TestConditionalFilter(c *C) {
	s.filter = condfilter.EDIDFilter("foo")
	c.Assert(s.filter.ConditionalFilter(), Equals, `[EDID=foo]`)
}
