// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: Huawei Inc.

package condfilter_test

import (
	"errors"

	. "gopkg.in/check.v1"

	"git.ostc-eu.org/OSTC/OHOS/components/sysota/picfg/condfilter"
)

type noneFilterSuite struct {
	filter condfilter.NoneFilter
}

var _ = Suite(&noneFilterSuite{})

func (s *noneFilterSuite) TestMarshalText(c *C) {
	text, err := s.filter.MarshalText()
	c.Assert(err, IsNil)
	c.Check(string(text), Equals, "[none]")
}

func (s *noneFilterSuite) TestUnmarshalText(c *C) {
	err := s.filter.UnmarshalText([]byte(`[none]`))
	c.Assert(err, IsNil)

	err = s.filter.UnmarshalText([]byte(`[foo`))
	c.Assert(err, ErrorMatches, `NoneFilter "\[foo" error: syntax error`)
	c.Check(errors.Is(err, condfilter.ErrSyntax), Equals, true)

	err = s.filter.UnmarshalText([]byte(`foo]`))
	c.Assert(err, ErrorMatches, `NoneFilter "foo\]" error: syntax error`)
	c.Check(errors.Is(err, condfilter.ErrSyntax), Equals, true)

	err = s.filter.UnmarshalText([]byte(`[foo]`))
	c.Assert(err, ErrorMatches, `NoneFilter "\[foo\]" error: different filter`)
	c.Check(errors.Is(err, condfilter.ErrDifferent), Equals, true)
}

func (s *noneFilterSuite) TestConditionalFilter(c *C) {
	c.Assert(s.filter.ConditionalFilter(), Equals, `[none]`)
}
