// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: Huawei Inc.

package condfilter_test

import (
	"errors"
	"strconv"

	. "gopkg.in/check.v1"

	"git.ostc-eu.org/OSTC/OHOS/components/sysota/picfg/condfilter"
)

type serialFilterSuite struct {
	filter condfilter.SerialFilter
}

var _ = Suite(&serialFilterSuite{})

func (s *serialFilterSuite) TestMarshalText(c *C) {
	serialFilter := condfilter.SerialFilter(0x1234)
	text, err := serialFilter.MarshalText()
	c.Assert(err, IsNil)
	c.Check(string(text), Equals, "[0x1234]")
}

func (s *serialFilterSuite) TestUnmarshalText(c *C) {
	var paramErr *condfilter.ParameterError

	err := s.filter.UnmarshalText([]byte(`[0x1234]`))
	c.Assert(err, IsNil)
	c.Check(s.filter, Equals, condfilter.SerialFilter(0x1234))

	err = s.filter.UnmarshalText([]byte(`[0x potato]`))
	c.Assert(err, ErrorMatches, `SerialFilter "\[0x potato\]" error: invalid CPU serial number: invalid syntax`)
	c.Check(errors.Is(err, strconv.ErrSyntax), Equals, true)
	c.Assert(errors.As(err, &paramErr), Equals, true)
	c.Check(paramErr.What, Equals, condfilter.CPUSerialNumber)

	err = s.filter.UnmarshalText([]byte(`[foo`))
	c.Assert(err, ErrorMatches, `SerialFilter "\[foo" error: syntax error`)
	c.Check(errors.Is(err, condfilter.ErrSyntax), Equals, true)

	err = s.filter.UnmarshalText([]byte(`foo]`))
	c.Assert(err, ErrorMatches, `SerialFilter "foo\]" error: syntax error`)
	c.Check(errors.Is(err, condfilter.ErrSyntax), Equals, true)

	err = s.filter.UnmarshalText([]byte(`[foo]`))
	c.Assert(err, ErrorMatches, `SerialFilter "\[foo\]" error: different filter`)
	c.Check(errors.Is(err, condfilter.ErrDifferent), Equals, true)
}

func (s *serialFilterSuite) TestConditionalFilter(c *C) {
	s.filter = condfilter.SerialFilter(0x1234)
	c.Assert(s.filter.ConditionalFilter(), Equals, `[0x1234]`)
}

func (s *serialFilterSuite) TestMatchesSerial(c *C) {
	s.filter = condfilter.SerialFilter(0x1234)

	c.Check(s.filter.MatchesSerial(0x1234), Equals, true)
	c.Check(s.filter.MatchesSerial(0x5671), Equals, false)
}
