// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: Huawei Inc.

package pimodel_test

import (
	. "gopkg.in/check.v1"

	"git.ostc-eu.org/OSTC/OHOS/components/sysota/picfg/pimodel"
)

type socSuite struct{}

var _ = Suite(&socSuite{})

func (s *socSuite) TestString(c *C) {
	c.Check(pimodel.InvalidSoC.String(), Equals, "invalid")

	c.Check(pimodel.BCM2835.String(), Equals, "BCM2835")
	c.Check(pimodel.BCM2836.String(), Equals, "BCM2836")
	c.Check(pimodel.BCM2837.String(), Equals, "BCM2837")
	c.Check(pimodel.BCM2837B0.String(), Equals, "BCM2837B0")
	c.Check(pimodel.BCM2711.String(), Equals, "BCM2711")

	c.Check(func() { _ = pimodel.SoC(42).String() }, PanicMatches, "cannot find name of SoC 0x2a")
}

func (s *socSuite) TestBcmSiliconFamilyCode(c *C) {
	c.Check(pimodel.BCM2835.BcmSiliconFamilyCode(), Equals, 2708)
	c.Check(pimodel.BCM2836.BcmSiliconFamilyCode(), Equals, 2709)
	c.Check(pimodel.BCM2837.BcmSiliconFamilyCode(), Equals, 2710)
	c.Check(pimodel.BCM2837B0.BcmSiliconFamilyCode(), Equals, 2710)
	c.Check(pimodel.BCM2711.BcmSiliconFamilyCode(), Equals, 2711)

	c.Check(func() { _ = pimodel.SoC(42).BcmSiliconFamilyCode() }, PanicMatches, "cannot find silicon family code of SoC 0x2a")
}

func (s *socSuite) TestBcmPackageCode(c *C) {
	c.Check(pimodel.BCM2835.BcmPackageCode(), Equals, 2835)
	c.Check(pimodel.BCM2836.BcmPackageCode(), Equals, 2836)
	c.Check(pimodel.BCM2837.BcmPackageCode(), Equals, 2837)
	c.Check(pimodel.BCM2837B0.BcmPackageCode(), Equals, 2837)
	c.Check(pimodel.BCM2711.BcmPackageCode(), Equals, 2711)

	c.Check(func() { _ = pimodel.SoC(42).BcmPackageCode() }, PanicMatches, "cannot find package code of SoC 0x2a")
}
