// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: Huawei Inc.

package rauctest_test

import (
	"errors"

	"git.ostc-eu.org/OSTC/OHOS/components/sysota/rauc/installhandler"
	"git.ostc-eu.org/OSTC/OHOS/components/sysota/rauc/rauctest"
	. "gopkg.in/check.v1"
)

type installParticipantSuite struct{}

var _ = Suite(&installParticipantSuite{})

var errBoom = errors.New("boom")

func (s *installParticipantSuite) TestPreInstallHandler(c *C) {
	p := rauctest.InstallParticipant{}
	ctx := installhandler.HandlerContext{}
	called := false

	c.Check(func() { _ = p.PreInstallHandler(&ctx) }, PanicMatches, `please provide PreInstallHandlerFn callback function`)

	p.PreInstallHandlerFn = func(ctx *installhandler.HandlerContext) error {
		called = true

		return nil
	}

	err := p.PreInstallHandler(&ctx)
	c.Assert(err, IsNil)
	c.Check(called, Equals, true)

	p.PreInstallHandlerFn = func(ctx *installhandler.HandlerContext) error {
		return errBoom
	}

	err = p.PreInstallHandler(&ctx)
	c.Assert(err, ErrorMatches, `boom`)
}

func (s *installParticipantSuite) TestPostInstallHandler(c *C) {
	p := rauctest.InstallParticipant{}
	ctx := installhandler.HandlerContext{}
	called := false

	c.Check(func() { _ = p.PostInstallHandler(&ctx) }, PanicMatches, `please provide PostInstallHandlerFn callback function`)

	p.PostInstallHandlerFn = func(ctx *installhandler.HandlerContext) error {
		called = true

		return nil
	}

	err := p.PostInstallHandler(&ctx)
	c.Assert(err, IsNil)
	c.Check(called, Equals, true)

	p.PostInstallHandlerFn = func(ctx *installhandler.HandlerContext) error {
		return errBoom
	}

	err = p.PostInstallHandler(&ctx)
	c.Assert(err, ErrorMatches, `boom`)
}

func (s *installParticipantSuite) TestResetCallbacks(c *C) {
	p := rauctest.InstallParticipant{
		PreInstallHandlerFn:  func(ctx *installhandler.HandlerContext) error { return nil },
		PostInstallHandlerFn: func(ctx *installhandler.HandlerContext) error { return nil },
	}

	p.ResetCallbacks()

	c.Check(p, DeepEquals, rauctest.InstallParticipant{})
}
