// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: Huawei Inc.

package squashgzip_test

import (
	"testing"

	. "gopkg.in/check.v1"

	"git.ostc-eu.org/OSTC/OHOS/components/sysota/squashfstools/mksquashfs/squashgzip"
)

func Test(t *testing.T) { TestingT(t) }

type configSuite struct {
	config squashgzip.Config
}

var _ = Suite(&configSuite{})

func (s *configSuite) SetUpTest(c *C) {
	s.config = squashgzip.Config{}
}

func (s *configSuite) TestCompressionName(c *C) {
	c.Check(s.config.CompressionName(), Equals, "gzip")
}

func (s *configSuite) TestMksquashfsOptionsDefaults(c *C) {
	c.Check(s.config.MksquashfsOptions(), HasLen, 0)
}

func (s *configSuite) TestMksquashfsOptionsWithCompressionLevel(c *C) {
	s.config.CompressionLevel = 9
	c.Check(s.config.MksquashfsOptions(), DeepEquals, []string{"-X" + "compression-level", "9"})
}

func (s *configSuite) TestMksquashfsOptionsWithWindowSize(c *C) {
	s.config.WindowSize = 15
	c.Check(s.config.MksquashfsOptions(), DeepEquals, []string{"-X" + "window-size", "15"})
}

func (s *configSuite) TestMksquashfsOptionsWithStrategy(c *C) {
	s.config.Strategy = []squashgzip.Strategy{
		squashgzip.Filtered,
		squashgzip.RunLengthEncoded,
	}
	c.Check(s.config.MksquashfsOptions(), DeepEquals, []string{"-X" + "strategy", "filtered,run_length_encoded"})
}

type strategySuite struct{}

var _ = Suite(&strategySuite{})

func (s *strategySuite) TestString(c *C) {
	c.Check(squashgzip.Default.String(), Equals, "default")
	c.Check(squashgzip.Filtered.String(), Equals, "filtered")
	c.Check(squashgzip.HuffmanOnly.String(), Equals, "huffman_only")
	c.Check(squashgzip.RunLengthEncoded.String(), Equals, "run_length_encoded")
	c.Check(squashgzip.Fixed.String(), Equals, "fixed")
	c.Check(func() { _ = squashgzip.Strategy(42).String() }, PanicMatches, "invalid gzip strategy: 42")
}
