// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: Huawei Inc.

package squashlzo_test

import (
	"testing"

	. "gopkg.in/check.v1"

	"git.ostc-eu.org/OSTC/OHOS/components/sysota/squashfstools/mksquashfs/squashlzo"
)

func Test(t *testing.T) { TestingT(t) }

type configSuite struct {
	config squashlzo.Config
}

var _ = Suite(&configSuite{})

func (s *configSuite) SetUpTest(c *C) {
	s.config = squashlzo.Config{}
}

func (s *configSuite) TestCompressionName(c *C) {
	c.Check(s.config.CompressionName(), Equals, "lzo")
}

func (s *configSuite) TestMksquashfsOptionsDefaults(c *C) {
	c.Check(s.config.MksquashfsOptions(), HasLen, 0)
}

func (s *configSuite) TestMksquashfsOptionsWithCompressionLevel(c *C) {
	s.config.Algorithm = squashlzo.Lzo1x999
	s.config.CompressionLevel = 9
	c.Check(s.config.MksquashfsOptions(), DeepEquals, []string{"-X" + "compression-level", "9"})

	// Compression level only applies to Lzo1x_999.
	s.config.Algorithm = squashlzo.Lzo1x1
	c.Check(s.config.MksquashfsOptions(), DeepEquals, []string{"-X" + "algorithm", "lzo1x_1"})
}

func (s *configSuite) TestMksquashfsOptionsWithAlgorithm(c *C) {
	s.config.Algorithm = squashlzo.Lzo1x1type12
	c.Check(s.config.MksquashfsOptions(), DeepEquals, []string{"-X" + "algorithm", "lzo1x_1_12"})
}

type algorithmSuite struct{}

var _ = Suite(&algorithmSuite{})

func (s *algorithmSuite) TestString(c *C) {
	c.Check(squashlzo.Lzo1x1.String(), Equals, "lzo1x_1")
	c.Check(squashlzo.Lzo1x1type11.String(), Equals, "lzo1x_1_11")
	c.Check(squashlzo.Lzo1x1type12.String(), Equals, "lzo1x_1_12")
	c.Check(squashlzo.Lzo1x1type15.String(), Equals, "lzo1x_1_15")
	c.Check(squashlzo.Lzo1x999.String(), Equals, "lzo1x_999")
	c.Check(func() { _ = squashlzo.Algorithm(42).String() }, PanicMatches, "invalid LZO algorithm: 42")
}
