// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: Huawei Inc.

package squashxz_test

import (
	"testing"

	. "gopkg.in/check.v1"

	"git.ostc-eu.org/OSTC/OHOS/components/sysota/squashfstools/mksquashfs/squashxz"
)

func Test(t *testing.T) { TestingT(t) }

type configSuite struct {
	config squashxz.Config
}

var _ = Suite(&configSuite{})

func (s *configSuite) SetUpTest(c *C) {
	s.config = squashxz.Config{}
}

func (s *configSuite) TestCompressionName(c *C) {
	c.Check(s.config.CompressionName(), Equals, "xz")
}

func (s *configSuite) TestMksquashfsOptionsDefaults(c *C) {
	c.Check(s.config.MksquashfsOptions(), HasLen, 0)
}

func (s *configSuite) TestMksquashfsOptionsWithDictionarySize(c *C) {
	s.config.DictionarySize = "50%"
	c.Check(s.config.MksquashfsOptions(), DeepEquals, []string{"-X" + "dict-size", "50%"})

	s.config.DictionarySize = "16K"
	c.Check(s.config.MksquashfsOptions(), DeepEquals, []string{"-X" + "dict-size", "16K"})
}

func (s *configSuite) TestMksquashfsOptionsWithCompressionFilters(c *C) {
	s.config.CompressionFilters = []string{"x86", "arm"}
	c.Check(s.config.MksquashfsOptions(), DeepEquals, []string{"-X" + "bcj", "x86,arm"})
}
